/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

//
// Created by webpigeon on 23/04/22.
//

#include "fggl/scenes/game.hpp"
#include "fggl/util/service.hpp"

#if __has_include("fggl/phys/bullet/bullet.hpp")
	#include "fggl/phys/bullet/bullet.hpp"
#endif

namespace fggl::scenes {

	Game::Game(fggl::App &app) : AppState(app) {
		m_input = app.services()->getInput();
	}

	void Game::activate() {
		fggl::AppState::activate();

		// setup the scene
		m_world = std::make_unique<ecs3::World>(*m_owner.registry());

		#ifdef FGGL_MODULE_BULLET
			// FIXME this ties bullet to the game state - which shouldn't be the case
			m_phys = std::make_unique<fggl::phys::bullet::BulletPhysicsEngine>(m_world.get());
		#endif
	}

	void Game::deactivate() {
		m_phys.reset();
		m_world.reset();
	}

	void Game::update() {
		assert( m_world && "called game update, but there was no world - was activate called?" );

		if ( m_input != nullptr ) {
			bool escapePressed = m_input->keyboard.pressed(glfwGetKeyScancode(GLFW_KEY_ESCAPE));
			if (escapePressed) {
				m_owner.change_state(m_previous);
			}
		}

		if ( m_phys != nullptr ) {
			m_phys->step();
		}

		m_world->reapEntities();
	}

	void Game::render(fggl::gfx::Graphics &gfx) {
		if ( m_world != nullptr ) {
			gfx.drawScene( *m_world );
		}
	}

} // namespace demo

